#pragma once
#include <functional>
#include <vector>
#include <Eigen/Core>
#include <Eigen/Dense>
#include "fem_solve.hpp"
#include "L2_norm.hpp"
#include "H1_norm.hpp"
#include "writer.hpp"
#include <igl/readMESH.h>

//! Compute the L2-error for different meshes.
//! 
//! @param baseMeshName the basename for the mesh (eg. Square).
//!                     the method will automatically append _<n>.stl
//!
//! @param maxLevel the max mesh level to use. Will read up to and including
//!                 baseMeshName_<maxLevel>.stl
//!
//! @param f the function f as in the exercise (LHS)
//! @param g the function g as in the exercise (boundary value)
//! @param exactSolution the exact solution (found by eg. hand computation or googling).
void convergenceAnalysis(const std::string& baseMeshName, int maxLevel,
    const std::function<double(double, double)> f,
    const std::function<double(double, double)> g,
			 const std::function<double(double, double)> exactSolution,const std::function<Eigen::Vector2d(double, double)> exactSolution_grad)
{
    std::vector<double> differences_L2;
    std::vector<double> differences_H1;
    std::vector<int> numberOfDegreesOfFreedom;
    for (int i = 0; i <= maxLevel; i++) {
        Vector u;
        Eigen::MatrixXd vertices;
        Eigen::MatrixXi triangles;
        Eigen::MatrixXi tetrahedra;

        std::stringstream basenameSS;
        basenameSS << baseMeshName << "_" << i;
        std::string basename = basenameSS.str();

        igl::readMESH(std::string(CMEA2_DATA_PATH)
            + basename + ".mesh", vertices, tetrahedra, triangles);
        std::cout << "Computing convergence. At: " << basename << std::endl;
        const int degreeOfFreedom = solveFiniteElement(u, vertices, triangles, f, g);

        differences_L2.push_back(computeL2Difference(vertices, triangles, u, exactSolution));
	differences_H1.push_back(computeH1Difference(vertices, triangles, u, exactSolution_grad));
        numberOfDegreesOfFreedom.push_back(degreeOfFreedom);

        writeToFile(basename + "_values.txt", u);
        writeMatrixToFile(basename + "_vertices.txt", vertices);
        writeMatrixToFile(basename + "_triangles.txt", triangles);
    }

    std::stringstream errorL2Filename;
    errorL2Filename << baseMeshName << "_errorsL2.txt";

    std::stringstream errorH1Filename;
    errorH1Filename << baseMeshName << "_errorsH1.txt";

    writeToFile(errorL2Filename.str(), differences_L2);
    writeToFile(errorH1Filename.str(), differences_H1);
    writeToFile(baseMeshName + "_resolutions.txt", numberOfDegreesOfFreedom);
}
